﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using Gurux.Common;
using System.Xml.Serialization;

namespace Gurux.GXOwnMedia
{
    /// <summary>
    /// Gurux Own media.
    /// </summary>
    public class GXOwnMedia : Gurux.Common.IGXMedia
    {

        /// <summary>
        /// IP Address field what our media component uses.
        /// </summary>
        /// <remarks>
        /// This is a only example property for our own media component.
        /// </remarks>
        public string IPAddress
        {
            get;
            set;
        }
        /// <summary>
        /// Is media open.
        /// </summary>
        bool open = false;

        /// <summary>
        /// Is data send syncronously.
        /// </summary>
        readonly object synchronous = new object();

        /// <summary>
        /// Sync object.
        /// </summary>
        private object sync = new object();

        public ulong BytesReceived
        {
            get
            {
                return 0;
            }
        }

        public ulong BytesSent
        {
            get
            {
                return 0;
            }
        }

        public int ConfigurableSettings
        {
            get
            {
                return 0;
            }
            set
            {
            }
        }

        /// <summary>
        /// This component is enabled.
        /// </summary>
        public bool Enabled
        {
            get
            {
                return true;
            }
        }

        public object Eop
        {
            get
            {
                throw new NotImplementedException();
            }

            set
            {
                throw new NotImplementedException();
            }
        }

        //Check is media open.
        public bool IsOpen
        {
            get
            {
                return open;
            }
        }

        public bool IsSynchronous
        {
            get
            {
                bool reserved = System.Threading.Monitor.TryEnter(synchronous, 0);
                if (reserved)
                {
                    System.Threading.Monitor.Exit(synchronous);
                }
                return !reserved;
            }
        }

        /// <inheritdoc cref="IGXMedia.MediaContainer"/>
        IGXMediaContainer IGXMedia.MediaContainer
        {
            get;
            set;
        }

        /// <inheritdoc cref="IGXMedia.MediaType"/>
        public string MediaType
        {
            get
            {
                return "OwnMedia";
            }
        }

        /// <inheritdoc cref="IGXMedia.Name"/>
        public string Name
        {
            get
            {
                return IPAddress;
            }
        }

        public System.Windows.Forms.Form PropertiesForm
        {
            get
            {
                return new GXOwnMediaProperties(this);
            }
        }

        /// <summary>
        /// This serializes all properties from the component. 
        /// </summary>
        /// <remarks>
        /// Modify this for your needs.
        /// </remarks>
        public string Settings
        {
            get
            {
                return IPAddress;
            }
            set
            {
                IPAddress = value;
            }
        }

        public object Synchronous
        {
            get
            {
                return synchronous;
            }
        }

        public object SyncRoot
        {
            get
            {
                //In some special cases when binary serialization is used this might be null
                //after deserialize. Just set it.
                if (sync == null)
                {
                    sync = new object();
                }
                return sync;
            }
        }

        public object Tag
        {
            get;
            set;
        }

        public TraceLevel Trace
        {
            get;
            set;
        }

        public event ClientConnectedEventHandler OnClientConnected;
        public event ClientDisconnectedEventHandler OnClientDisconnected;
        public event ErrorEventHandler OnError;
        public event MediaStateChangeEventHandler OnMediaStateChange;
        public event ReceivedEventHandler OnReceived;
        public event TraceEventHandler OnTrace;

        public void Close()
        {
            open = false;
        }

        public void Copy(object target)
        {
            throw new NotImplementedException();
        }

        public void Open()
        {
            open = true;
        }

        public bool Properties(System.Windows.Forms.Form parent)
        {
            throw new NotImplementedException();
        }

        public bool Receive<T>(ReceiveParameters<T> args)
        {
            return false;
        }

        public void ResetByteCounters()
        {

        }

        public void ResetSynchronousBuffer()
        {

        }

        public void Send(object data, string receiver)
        {
            //Send data to the meter.
        }

        public void Validate()
        {
            if (string.IsNullOrEmpty(IPAddress))
            {
                throw new Exception("Invalid IP address.");
            }
        }
    }
}
